require "cut_scene_common"
require "game_common"
require "lang_str"

function show_bird()
  
  WaitTime(2)

  local bird_z = 0.3

  while true do

    -- 1.
    -- left-bottom -> right-top
    --
    if bird then cut_scene.ObjRemove(scene, bird) end
    bird = SpawnParticleSystem("media/complete/bird.xml", "", 0, 0)

    cut_scene.ObjSetPos(bird, -350, 0, bird_z)
    MoveTo(bird, -120, 100, bird_z, 5)
    MoveTo(bird, 350, 300, bird_z, 8.5)

    WaitTime(1)

    -- 2.
    -- right-center -> center-bottom
    --
    if bird then cut_scene.ObjRemove(scene, bird) end
    bird = SpawnParticleSystem("media/complete/bird.xml", "", 0, 0)

    cut_scene.ObjSetPos(bird, 350, 80, bird_z)
    MoveTo(bird, -80, -220, bird_z, 7)

    WaitTime(2)

    -- 3.
    -- right-center -> left-top
    --
    if bird then cut_scene.ObjRemove(scene, bird) end
    bird = SpawnParticleSystem("media/complete/bird.xml", "", 0, 0)

    cut_scene.ObjSetPos(bird, 350, -100, bird_z)
    MoveTo(bird, 90, 30, bird_z, 5)
    MoveTo(bird, -350, 200, bird_z, 8)

    WaitTime(3)

  end
end

room_x = 0
room_y = 0
first_room_noise = true

function room_noise()

  WaitTime(2)

  while true do

    local wait_period = 5 + math.random() * 5
    if first_room_noise then
      wait_period = math.random()
      first_room_noise = false
    end

    WaitTime(wait_period)

    local num = math.random(4)
    for i = 1, num do

      SpawnParticleSystem("media/complete/room_noise.xml", "", room_x, room_y + 40, 0.3)

      if math.random() > 0.25 then
        cut_scene.Sfx("media/audio/hit2.caf", false, 0.5 + math.random() * 0.25, 0.75 + math.random() * 0.25)
      else
        cut_scene.Sfx("media/audio/succ1.caf", false, 0.5 + math.random() * 0.25, 0.75 + math.random() * 0.25)
      end

      WaitTime(0.25)

    end

  end
end

function white_mask_out()
  local white_mask = SpawnSprite("", 568, 384, 0, 0, 1)
  cut_scene.ObjSetColor(white_mask, { 255, 255, 255, 255 })
  TransparentTo(white_mask, 0, 3, true)
  cut_scene.ObjRemove(scene, white_mask)
end

stamp_studio_pos = { x = 150, y = 96 }

function start()
  
  game.InkOut()

  local complete_difficulty = game.CompleteDifficulty()
  if complete_difficulty == DIFFICULTY_MAX then
    complete_difficulty = DIFFICULTY_EASY
  end

  -- test usage
  -- complete_difficulty = DIFFICULTY_NORMAL

  local fg_res = "media/complete/bg_easy.png"
  local fg_x = 0
  local fg_y = 0

  local stamp_difficulty_unit
  local stamp_difficulty_pos = { x = 0, y = 0 }

  local lang_type = lang.GetLangType()
  local use_eng_stamp = (lang_type ~= LANG_ZHTW and lang_type ~= LANG_ZHCN and lang_type ~= LANG_JA)

  if DIFFICULTY_EASY == complete_difficulty then

    fg_res = "media/complete/bg_easy.png"
    fg_y = 5
    room_x = 55
    room_y = 10
    stamp_difficulty_unit = lang.GetLangStr(LS_STAMP_EASY)
    -- stamp_studio_pos.x = 150
    -- stamp_studio_pos.y = 96

    if use_eng_stamp then
      stamp_difficulty_pos.x = -195
      stamp_difficulty_pos.y = -40
    else
      stamp_difficulty_pos.x = -190
      stamp_difficulty_pos.y = -55
    end

  elseif DIFFICULTY_NORMAL == complete_difficulty then

    fg_res = "media/complete/bg_normal.png"
    fg_y = 5
    room_x = 38
    room_y = -20
    stamp_difficulty_unit = lang.GetLangStr(LS_STAMP_NORMAL)
    -- stamp_studio_pos.x = 150
    stamp_studio_pos.y = 90

    if use_eng_stamp then
      stamp_difficulty_pos.x = -170
      stamp_difficulty_pos.y = -60
    else
      stamp_difficulty_pos.x = -160
      stamp_difficulty_pos.y = -60
    end

  elseif DIFFICULTY_HARD == complete_difficulty then

    fg_res = "media/complete/bg_hard.png"
    room_x = 8
    room_y = -45
    stamp_difficulty_unit = lang.GetLangStr(LS_STAMP_HARD)
    -- stamp_studio_pos.x = 150
    stamp_studio_pos.y = 75

    if use_eng_stamp then
      stamp_difficulty_pos.x = -150
      stamp_difficulty_pos.y = -80
    else
      stamp_difficulty_pos.x = -150
      stamp_difficulty_pos.y = -90
    end

  end

  local bg = SpawnSprite("media/common/bg0.png", 568, 384, 0, 0, 0)
  local fg = SpawnSprite(fg_res, 568, 384, fg_x, fg_y, 0.1)
  --
  local room = SpawnSprite("", 44, 44, room_x, room_y, 0.2, "room")
  cut_scene.ObjSetColor(room, { 255, 0, 0, 128 })
  cut_scene.ObjSetVisible(room, false)

  --
  game.ShowPauseBtn()
  white_mask_out()

  --
  cut_scene.CreateCoroutine(scene, "show_bird", "show_bird_co")
  cut_scene.CreateCoroutine(scene, "room_noise", "room_noise_co")

  --
  local stamp_studio = SpawnSpriteAtlas(
    "media/complete/stamp_actions.png", "media/complete/stamp_actions", "stamp_studio.png",
    0.5, stamp_studio_pos.x, stamp_studio_pos.y, 1, "studio") -- 25 x 31
  cut_scene.ObjSetAreaBorder(stamp_studio, 10, 7)
  cut_scene.ObjSetColor(stamp_studio, { 255, 255, 255, 0})
  TransparentTo(stamp_studio, 255, 2, false)

  --
  local stamp_difficulty = SpawnSpriteAtlas(
    "media/complete/stamp_difficulty.png", "media/complete/stamp_difficulty", stamp_difficulty_unit,
    0.5, stamp_difficulty_pos.x, stamp_difficulty_pos.y, 1, "difficulty")
  cut_scene.ObjSetColor(stamp_difficulty, { 255, 255, 255, 0})
  TransparentTo(stamp_difficulty, 255, 2, false)

end

--------------------------------------------------------------------------------

stamp_rate = nil
stamp_twitter = nil
stamp_facebook = nil
stamp_aride = nil
action_txt = nil
action_yes = nil
action_no = nil

function show_actions()

  if stamp_rate == nil then

    pos_x = stamp_studio_pos.x - 32
    pos_y = stamp_studio_pos.y - 38

    stamp_rate = SpawnSpriteAtlas(
      "media/complete/stamp_actions.png", "media/complete/stamp_actions", "stamp_rate.png",
      0.5, pos_x, pos_y, 1) -- 33 x 39
    cut_scene.ObjSetAreaBorder(stamp_rate, 6, 3)
    cut_scene.ObjSetColor(stamp_rate, { 255, 255, 255, 0})

    pos_x = stamp_studio_pos.x + 4
    pos_y = stamp_studio_pos.y - 70

    stamp_twitter = SpawnSpriteAtlas(
      "media/complete/stamp_actions.png", "media/complete/stamp_actions", "stamp_twitter.png",
      0.5, pos_x, pos_y, 1) -- 34 x 40
    cut_scene.ObjSetAreaBorder(stamp_twitter, 5, 2)
    cut_scene.ObjSetColor(stamp_twitter, { 255, 255, 255, 0})

    pos_x = stamp_studio_pos.x + 44
    pos_y = stamp_studio_pos.y + 7

    stamp_facebook = SpawnSpriteAtlas(
      "media/complete/stamp_actions.png", "media/complete/stamp_actions", "stamp_facebook.png",
      0.5, pos_x, pos_y, 1) -- 33 x 39
    cut_scene.ObjSetAreaBorder(stamp_facebook, 6, 3)
    cut_scene.ObjSetColor(stamp_facebook, { 255, 255, 255, 0})

    pos_x = stamp_studio_pos.x + 54
    pos_y = stamp_studio_pos.y - 55

    stamp_aride = SpawnSpriteAtlas(
      "media/complete/stamp_actions.png", "media/complete/stamp_actions", "stamp_aride.png",
      0.5, pos_x, pos_y, 1) -- 36 x 53
    cut_scene.ObjSetAreaBorder(stamp_aride, 4, 0)
    cut_scene.ObjSetColor(stamp_aride, { 255, 255, 255, 0})

    --

    action_txt = SpawnTxt("", DEFAULT_FONT, DEFAULT_FONT_SIZE, -90, 104, 1)
    cut_scene.ObjSetColor(action_txt, { 0, 0, 0, 0 })

    action_yes = SpawnTxt(lang.GetLangStr(LS_COMPLETE_YES), DEFAULT_FONT, DEFAULT_FONT_SIZE, -125, 44, 1)
    cut_scene.ObjSetAreaBorder(action_yes, 15, 15)
    cut_scene.ObjSetColor(action_yes, { 0, 0, 0, 0 })

    action_no = SpawnTxt(lang.GetLangStr(LS_COMPLETE_NO), DEFAULT_FONT, DEFAULT_FONT_SIZE, -55, 44, 1)
    cut_scene.ObjSetAreaBorder(action_no, 15, 15)
    cut_scene.ObjSetColor(action_no, { 0, 0, 0, 0 })

  end

  TransparentTo(stamp_rate, 255, 0.5, false)
  WaitTime(0.1)
  TransparentTo(stamp_twitter, 255, 0.5, false)
  WaitTime(0.1)
  TransparentTo(stamp_facebook, 255, 0.5, false)
  WaitTime(0.1)
  TransparentTo(stamp_aride, 255, 0.5, false)

  cut_scene.ObjSetName(scene, stamp_rate, "rate")
  cut_scene.ObjSetName(scene, stamp_twitter, "twitter")
  cut_scene.ObjSetName(scene, stamp_facebook, "facebook")
  cut_scene.ObjSetName(scene, stamp_aride, "aride")
  cut_scene.ObjSetName(scene, action_yes, "yes")
  cut_scene.ObjSetName(scene, action_no, "no")
end

function hide_actions()

  if stamp_rate == nil then
    return
  end

  cut_scene.ObjSetName(scene, stamp_rate, "")
  cut_scene.ObjSetName(scene, stamp_twitter, "")
  cut_scene.ObjSetName(scene, stamp_facebook, "")
  cut_scene.ObjSetName(scene, stamp_aride, "")
  cut_scene.ObjSetName(scene, action_yes, "")
  cut_scene.ObjSetName(scene, action_no, "")

  ColorTo(action_txt, { 0, 0, 0, 0 }, 0.5, false)
  ColorTo(action_yes, { 0, 0, 0, 0 }, 0.5, false)
  ColorTo(action_no, { 0, 0, 0, 0 }, 0.5, false)

  TransparentTo(stamp_aride, 0, 0.5, false)
  WaitTime(0.1)
  TransparentTo(stamp_twitter, 0, 0.5, false)
  WaitTime(0.1)
  TransparentTo(stamp_facebook, 0, 0.5, false)
  WaitTime(0.1)
  TransparentTo(stamp_rate, 0, 0.5, false)
  WaitTime(0.6)

  cut_scene.ObjRemove(scene, stamp_rate)
  cut_scene.ObjRemove(scene, stamp_twitter)
  cut_scene.ObjRemove(scene, stamp_facebook)
  cut_scene.ObjRemove(scene, stamp_aride)
  stamp_rate = nil
  stamp_twitter = nil
  stamp_facebook = nil
  stamp_aride = nil

  cut_scene.ObjRemove(scene, action_txt)
  cut_scene.ObjRemove(scene, action_yes)
  cut_scene.ObjRemove(scene, action_no)
  action_txt = nil
  action_yes = nil
  action_no = nil
end

actions_visible = false

function on_studio()
  actions_visible = not actions_visible

  if actions_visible then
    show_actions()
  else
    hide_actions()
  end
end

function show_action_confirm(msg)
  cut_scene.TxtSetMsg(action_txt, msg)
  cut_scene.ObjSetColor(action_txt, { 0, 0, 0, 0 })
  ColorTo(action_txt, { 0, 0, 0, 255 }, 0.5, false)

  cut_scene.ObjSetColor(action_yes, { 0, 0, 0, 0 })
  ColorTo(action_yes, { 0, 0, 0, 255 }, 0.5, false)
  cut_scene.ObjSetColor(action_no, { 0, 0, 0, 0 })
  ColorTo(action_no, { 0, 0, 0, 255 }, 0.5, false)
end

function hide_action_confirm()
  ColorTo(action_txt, { 0, 0, 0, 0 }, 0.5, false)
  ColorTo(action_yes, { 0, 0, 0, 0 }, 0.5, false)
  ColorTo(action_no, { 0, 0, 0, 0 }, 0.5, false)
end

current_action = ""

function on_rate()
  current_action = "rate"
  show_action_confirm(lang.GetLangStr(LS_COMPLETE_RATE))
end

function on_twitter()
  current_action = "twitter"
  show_action_confirm(lang.GetLangStr(LS_COMPLETE_TWITTER))
end

function on_facebook()
  current_action = "facebook"
  show_action_confirm(lang.GetLangStr(LS_COMPLETE_FACEBOOK))
end

function on_aride()
  current_action = "aride"
  show_action_confirm(lang.GetLangStr(LS_COMPLETE_ARIDE))
end

function on_yes()
  complete_lib.Action(current_action)
  hide_action_confirm()
end

function on_no()
  hide_action_confirm()
end
