require "cut_scene_common"
require "game_common"
require "lang_str"

can_leave = true
is_leaving = false

function start()

  bg = SpawnSprite("media/common/bg5.png", 568, 384, 0, 0, 10)
  fg = SpawnSprite("media/complete/room_fg.png", 568, 384, 0, 0, 10.1)
  cut_scene.ObjSetColor(bg, { 255, 255, 255, 0 })
  cut_scene.ObjSetColor(fg, { 255, 255, 255, 0 })
  TransparentTo(bg, 255, 0.2, false)
  TransparentTo(fg, 255, 0.2)

  --
  learn_sword = SpawnSprite("", 155, 45, -10, 40, 11, "learn")
  cut_scene.ObjSetColor(learn_sword, { 255, 0, 0, 128 })
  cut_scene.ObjSetVisible(learn_sword, false)
  --
  sharp_sword = SpawnSprite("", 160, 45, 15, -5, 11, "sharp")
  cut_scene.ObjSetColor(sharp_sword, { 255, 0, 0, 64 })
  cut_scene.ObjSetVisible(sharp_sword, false)
  --
  break_sword = SpawnSprite("", 185, 45, -115, -50, 11, "break")
  cut_scene.ObjSetColor(break_sword, { 0, 255, 0, 128 })
  cut_scene.ObjSetVisible(break_sword, false)
  --
  heavy_sword = SpawnSprite("", 250, 50, -55, -100, 11, "heavy")
  cut_scene.ObjSetColor(heavy_sword, { 0, 255, 0, 64 })
  cut_scene.ObjSetVisible(heavy_sword, false)
  --
  wood_sword = SpawnSprite("", 150, 50, -165, 5, 11, "wood")
  cut_scene.ObjSetColor(wood_sword, { 0, 0, 255, 128 })
  cut_scene.ObjSetVisible(wood_sword, false)
  --
  none_sword = SpawnSprite("", 100, 100, 170, 75, 11, "none")
  cut_scene.ObjSetColor(none_sword, { 0, 0, 255, 64 })
  cut_scene.ObjSetVisible(none_sword, false)

  -- light_effect = SpawnParticleSystem("media/room/light.xml", "", 210, 100, 12)

  --

  info_txt = SpawnTxt("", DEFAULT_FONT, DEFAULT_FONT_SIZE, -90, 100, 12)
  cut_scene.ObjSetColor(info_txt, { 0, 0, 0, 0 })

  confirm_yes = SpawnTxt(lang.GetLangStr(LS_ROOM_YES), DEFAULT_FONT, DEFAULT_FONT_SIZE, 180, -50, 12)
  cut_scene.ObjSetAreaBorder(confirm_yes, 15, 15)
  cut_scene.ObjSetColor(confirm_yes, { 0, 0, 0, 0 })

  confirm_no = SpawnTxt(lang.GetLangStr(LS_ROOM_NO), DEFAULT_FONT, DEFAULT_FONT_SIZE, 180, -110, 12)
  cut_scene.ObjSetAreaBorder(confirm_no, 15, 15)
  cut_scene.ObjSetColor(confirm_no, { 0, 0, 0, 0 })

end

--------------------------------------------------------------------------------

function show_confirm(msg)
  cut_scene.ObjSetName(scene, confirm_yes, "yes")
  cut_scene.ObjSetName(scene, confirm_no, "no")

  cut_scene.TxtSetMsg(info_txt, msg)
  cut_scene.ObjSetColor(info_txt, { 0, 0, 0, 0 })
  ColorTo(info_txt, { 0, 0, 0, 255 }, 0.5, false)

  cut_scene.ObjSetColor(confirm_yes, { 0, 0, 0, 0 })
  ColorTo(confirm_yes, { 0, 0, 0, 255 }, 0.5, false)
  cut_scene.ObjSetColor(confirm_no, { 0, 0, 0, 0 })
  ColorTo(confirm_no, { 0, 0, 0, 255 }, 0.5, false)
end

function hide_confirm()
  cut_scene.ObjSetName(scene, confirm_yes, "")
  cut_scene.ObjSetName(scene, confirm_no, "")

  ColorTo(info_txt, { 0, 0, 0, 0 }, 0.5, false)
  ColorTo(confirm_yes, { 0, 0, 0, 0 }, 0.5, false)
  ColorTo(confirm_no, { 0, 0, 0, 0 }, 0.5, false)
end

function show_select(name, x, y)
  if select_sword then
    cut_scene.ObjRemove(scene, select_sword)
  end

  select_sword = SpawnSpriteAtlas("media/complete/select_sword.png", "media/complete/select_sword", name, 0.5, x, y, 10.2)
  cut_scene.ObjSetColor(select_sword, { 255, 255, 255, 0 })
  TransparentTo(select_sword, 255, 0.5, false)
end

function hide_select()
  if select_sword then
    hiding_select = select_sword
    select_sword = nil

    TransparentTo(hiding_select, 0, 0.5)
    cut_scene.ObjRemove(scene, hiding_select)
    hiding_select = nil
  end
end

current_stage = ""

function on_learn()
  current_stage = "learn"
  show_confirm(lang.GetLangStr(LS_ROOM_INFO_LEARN))
  show_select("learn.png", -38, 40)
end

function on_sharp()
  current_stage = "sharp"
  show_confirm(lang.GetLangStr(LS_ROOM_INFO_SHARP))
  show_select("sharp.png", 64, -2)
end

function on_break()
  current_stage = "break"
  show_confirm(lang.GetLangStr(LS_ROOM_INFO_BREAK))
  show_select("break.png", -155, -46)
end

function on_heavy()
  current_stage = "heavy"
  show_confirm(lang.GetLangStr(LS_ROOM_INFO_HEAVY))
  show_select("heavy.png", 18, -86)
end

function on_wood()
  current_stage = "wood"
  show_confirm(lang.GetLangStr(LS_ROOM_INFO_WOOD))
  show_select("wood.png", -132, 2)
end

function on_none()
  current_stage = "none"
  show_confirm(lang.GetLangStr(LS_ROOM_INFO_NONE))
  show_select("none.png", 156, 72)
end

function on_yes()
  is_leaving = true

  game.InkIn()
  cut_scene.StopBgm(1.5)
  WaitTime(1.5)
  game.GoMode(current_stage)
end

function on_no()
  hide_confirm()
  hide_select()
end

--------------------------------------------------------------------------------

function on_close()
  is_leaving = true

  -- cut_scene.ObjRemove(scene, light_effect)

  TransparentTo(bg, 0, 0.2, false)
  TransparentTo(fg, 0, 0.2, false)

  ColorTo(info_txt, { 0, 0, 0, 0 }, 0.2, false)
  ColorTo(confirm_yes, { 0, 0, 0, 0 }, 0.2, false)
  ColorTo(confirm_no, { 0, 0, 0, 0 }, 0.2, false)

  if select_sword then
    TransparentTo(select_sword, 0, 0.2, false)
  elseif hiding_select then
    TransparentTo(hiding_select, 0, 0.2, false)
  end

  WaitTime(0.3)

  game.PopState()
end
