GAME_STATE_MODE_SELECT    = 0
GAME_STATE_LEVEL_SELECT   = 1
GAME_STATE_LANGUAGE       = 2
GAME_STATE_DIFFICULTY     = 3
GAME_STATE_PAUSE_MENU     = 4
GAME_STATE_CONFIRM        = 5
GAME_STATE_CUT_SCENE      = 6
GAME_STATE_PLAY           = 7
GAME_STATE_SWORD_LEARN    = 8
GAME_STATE_SWORD_SHARP    = 9
GAME_STATE_SWORD_BREAK    = 10
GAME_STATE_SWORD_HEAVY    = 11
GAME_STATE_SWORD_WOOD     = 12
GAME_STATE_SWORD_NONE     = 13
GAME_STATE_SWORD_LEAF     = 14
GAME_STATE_SWORD_FLY      = 15
GAME_STATE_SWORD_ANIMAL   = 16
GAME_STATE_SWORD_BIRD     = 17
GAME_STATE_SWORD_SNOW     = 18
GAME_STATE_SWORD_CLIMB    = 19
GAME_STATE_SWORD_FLOWER   = 20
GAME_STATE_SWORD_CLOUD    = 21
GAME_STATE_SWORD_BALANCE  = 22
GAME_STATE_COMPLETE       = 23
GAME_STATE_ROOM           = 24

LANG_EN   = 0
LANG_ZHTW = 1
LANG_ZHCN = 2
LANG_JA   = 3
LANG_DE   = 4
LANG_FR   = 5
LANG_IT   = 6
LANG_ES   = 7

DIFFICULTY_HARD   = 0
DIFFICULTY_NORMAL = 1
DIFFICULTY_EASY   = 2
DIFFICULTY_MAX    = 3

DEFAULT_FONT = game.GetDefaultFont()
DEFAULT_FONT_SIZE = 20

function sword_opening_title(img, translate_id, translate_x, translate_y, translate_img_y_offset)

  game.InkIn(true)

  local lang_type = lang.GetLangType()
  local have_subtitle = (lang_type ~= LANG_ZHTW and lang_type ~= LANG_ZHCN and lang_type ~= LANG_JA)

  local img_y = 0
  if have_subtitle and translate_img_y_offset then
    img_y = translate_img_y_offset
  end

  local title_img = SpawnSprite(img, 511, 346, 0, img_y, 12) -- (568, 384) x 0.9
  -- cut_scene.SpriteAdjustSizeByHeight(title_img)
  cut_scene.ObjSetColor(title_img, { 255, 255, 255, 0 })
  TransparentTo(title_img, 255, 0.1, false)

  local translate = nil
  if have_subtitle then
    translate = SpawnTxt(lang.GetLangStr(translate_id), DEFAULT_FONT, DEFAULT_FONT_SIZE, translate_x, translate_y, 11)
    cut_scene.ObjSetColor(translate, { 0, 0, 0, 0 })
    ColorTo(translate, { 180, 180, 180, 255 }, 0.1, false)
  end

  WaitTime(1)
  WaitTimeAndClick(4)

  TransparentTo(title_img, 0, 0.2, false)
  if have_subtitle then
    ColorTo(translate, { 0, 0, 0, 0 }, 0.2, false)
  end

  WaitTime(0.5)

  cut_scene.ObjRemove(scene, title_img)
  if have_subtitle then
    cut_scene.ObjRemove(scene, translate)
  end

  game.InkOut()

end

function sword_opening(sequence, can_skip, no_txt_skip, after_sequence, no_ink_in, use_transition, txt_display_period)

  local images = {}
  local image_depth = 9
  local txt = nil

  for i,v in ipairs(sequence) do

    if v.img then -- img

      if #images == 0 then -- first

        local image = SpawnSprite(v.img, 568, 384, 0, 0, image_depth)
        table.insert(images, image)

        if use_transition then
          img_transition(image, image_depth, v)
        end

        if can_skip then
          WaitTimeAndClick(1.5)
        else
          WaitTime(1.5)
        end

      elseif string.len(v.img) == 0 then -- fade out

        local prev_image = images[#images]
        TransparentTo(prev_image, 0, 1, false)

        if can_skip then
          WaitTimeAndClick(1)
        else
          WaitTime(1)
        end

      else -- cross

        local new_image = SpawnSprite(v.img, 568, 384, 0, 0, image_depth)
        table.insert(images, new_image)
        cut_scene.ObjSetColor(new_image, { 255, 255, 255, 0 })
        TransparentTo(new_image, 255, 1, false)

        local prev_image = images[#images - 1]
        TransparentTo(prev_image, 0, 0.5, false)

        if use_transition then
          img_transition(new_image, image_depth, v)
        end

        if can_skip then
          WaitTimeAndClick(2)
        else
          WaitTime(2)
        end

        -- cut_scene.ObjRemove(scene, prev_image)

      end

      image_depth = image_depth + 0.1

    else -- txt

      if nil == txt then
        txt = SpawnTxt(lang.GetLangStr(v.translate_id), DEFAULT_FONT, DEFAULT_FONT_SIZE, v.x, v.y, 20)
      else
        cut_scene.TxtSetMsg(txt, lang.GetLangStr(v.translate_id))
        cut_scene.ObjSetPos(txt, v.x, v.y, 20)
      end

      cut_scene.ObjSetColor(txt, { 0, 0, 0, 0 })
      TransparentTo(txt, 255, 1, false)

      if nil == txt_display_period then txt_display_period = 2.5 end

      if can_skip then
        WaitTimeAndClick(txt_display_period)
      else
        if no_txt_skip then
          WaitTime(txt_display_period)
        else
          WaitTime(1.5)
          WaitTimeAndClick(txt_display_period - 1.5)
        end
      end
      
      TransparentTo(txt, 0, 0.5, false)

      if can_skip then
        WaitTimeAndClick(0.5)
      else
        WaitTime(0.5)
      end

    end

  end

  --

  if after_sequence then
    after_sequence()
  end

  --

  if no_ink_in then
    -- do nothing
  else
    game.InkIn()
    WaitTime(1)
  end

  -- clean up

  -- cut_scene.ObjRemove(scene, images[#images])
  for i,v in ipairs(images) do
    cut_scene.ObjRemove(scene, v)
  end

  if txt then
    cut_scene.ObjRemove(scene, txt)
  end

  -- release used textures

  for i,v in ipairs(sequence) do
    if v.img and string.len(v.img) > 0 then
      cut_scene.ReleaseTexture(v.img)
    end
  end

end

function sword_ending(img_setting, msgs, can_skip, use_transition, is_none)

  if game.IsDemo() then
    cut_scene.StopBgm(2)
    WaitTime(2)
    return
  end

  game.InkOut()

  local image = SpawnSprite(img_setting.img, 568, 384, 0, 0, 9)

  if use_transition then
    img_transition(image, 9, img_setting)
  end

  if can_skip then
    WaitTimeAndClick(1.5)
  else
    WaitTime(1.5)
  end

  local txt = nil

  for i,v in ipairs(msgs) do

    if nil == txt then
      txt = SpawnTxt(lang.GetLangStr(v.translate_id), DEFAULT_FONT, DEFAULT_FONT_SIZE, v.x, v.y, 20)
    else
      cut_scene.TxtSetMsg(txt, lang.GetLangStr(v.translate_id))
      cut_scene.ObjSetPos(txt, v.x, v.y, 20)
    end

    cut_scene.ObjSetColor(txt, { 0, 0, 0, 0 })

    local txt_fade_in_period = 1
    local txt_keep_period = 2
    local txt_fade_out_period = 0.5

    if is_none then
      txt_fade_in_period = 2
      txt_keep_period = 3
      txt_fade_out_period = 2
    end

    TransparentTo(txt, 255, txt_fade_in_period, false)

    if can_skip then
      WaitTimeAndClick(txt_fade_in_period + txt_keep_period)
    else
      WaitTime(txt_fade_in_period + 0.5)
      WaitTimeAndClick(txt_keep_period - 0.5)
    end

    TransparentTo(txt, 0, txt_fade_out_period, false)

    if can_skip then
      WaitTimeAndClick(txt_fade_out_period)
    else
      WaitTime(txt_fade_out_period)
    end

  end

  --

  local bgm_fade_out_period = 3.5
  local wait_ink_period = 2

  if is_none then
    bgm_fade_out_period = 5
    wait_ink_period =3
  end

  cut_scene.StopBgm(bgm_fade_out_period)

  if can_skip then
    WaitTimeAndClick(wait_ink_period)
    cut_scene.StopBgm(2)
  else
    WaitTime(wait_ink_period)
  end

  game.InkIn()
  WaitTime(2)

  -- clean up

  cut_scene.ObjRemove(scene, image)

  if txt then
    cut_scene.ObjRemove(scene, txt)
  end

  -- release used texture

  cut_scene.ReleaseTexture(img_setting.img)

end

function img_transition(img, depth, setting)

  local period = setting.transition_period
  if nil == period then period = 15 end

  if setting.move then
    local from = setting.move.from
    if nil == from then from = { 0, 0 } end

    local to = setting.move.to
    if nil == to then to = { 0, 0 } end

    if from[1] ~= to[1] or from[2] ~= to[2] then
      cut_scene.ObjSetPos(img, from[1], from[2], depth)
      MoveTo(img, to[1], to[2], depth, period, false)
    end
  end

  if setting.scale then
    local from = setting.scale.from
    if nil == from then from = 1 end

    local to = setting.scale.to
    if nil == to then to = 1 end

    if from ~= to then
      cut_scene.ObjSetScale(img, from)
      ScaleTo(img, to, to, period, false)
    end
  end

end
