#define NUM_TEXTURES 2

#define FOG_MODE_LINEAR 0
#define FOG_MODE_EXP    1
#define FOG_MODE_EXP2   2

#ifdef GL_ES
precision highp float;
precision lowp int;
#endif

const int i_zero = 0;
const int i_one = 1;

uniform mat4 model_view_proj_matrix;
uniform mat4 model_view_matrix;

#ifdef GL_ES
uniform lowp int tex_use_coord_idx[NUM_TEXTURES];
#else
uniform int tex_use_coord_idx[NUM_TEXTURES];
#endif

uniform bool fog_enable;
uniform float fog_density;
uniform float fog_start, fog_end;
uniform int   fog_mode;

attribute vec4 a_position;
attribute vec4 a_color;
attribute vec2 a_texcoord0;
attribute vec2 a_texcoord1;

varying vec4 v_color;
varying vec2 v_texcoord[NUM_TEXTURES];
varying float v_fog_factor;

vec4 p_eye;

float compute_fog()
{
  float f;
  if (fog_mode == FOG_MODE_LINEAR)
  {
    f = (fog_end + p_eye.z) / (fog_end - fog_start); // p_eye.z is negative
  }
  else if (fog_mode == FOG_MODE_EXP)
  {
    f = exp(-(p_eye.z * fog_density));
  }
  else if (fog_mode == FOG_MODE_EXP2)
  {
    f = p_eye.z * fog_density;
    f = exp(-(f * f));
  }

  f = clamp(f, 0.0, 1.0);
  return f;
}

void main()
{
  gl_Position = model_view_proj_matrix * a_position;

  v_color = a_color;

  if (tex_use_coord_idx[i_zero] == i_zero)
    v_texcoord[i_zero] = a_texcoord0;
  else if (tex_use_coord_idx[i_zero] == i_one)
    v_texcoord[i_zero] = a_texcoord1;

  if (tex_use_coord_idx[i_one] == i_zero)
    v_texcoord[i_one] = a_texcoord0;
  else if (tex_use_coord_idx[i_one] == i_one)
    v_texcoord[i_one] = a_texcoord1;

  if (fog_enable)
  {
    p_eye = model_view_matrix * a_position;
    v_fog_factor = compute_fog();
  }
}
