#define NUM_TEXTURES 2

#ifdef GL_ES
precision highp float;
precision lowp int;
#endif

const int i_zero = 0;
const int i_one = 1;

const float radius = 1.0;

uniform sampler2D tex[NUM_TEXTURES];
uniform float resolution;
uniform vec2 dir;

varying vec4 v_color;
varying vec2 v_texcoord[NUM_TEXTURES];

void main()
{
    vec4 sum = vec4(0.0);

    // our original texcoord for this fragment
    vec2 tc = v_texcoord[i_zero];

    // the amount to blur, i.e. how far off center to sample from 
    // 1.0 -> blur by one pixel
    // 2.0 -> blur by two pixels, etc.
    float blur = radius / resolution;

    float blur_hstep = blur * dir.x;
    float blur_vstep = blur * dir.y;

    // apply blurring, using a 9-tap filter with predefined gaussian weights

    sum += texture2D(tex[i_zero], vec2(tc.x - 4.0 * blur_hstep, tc.y - 4.0 * blur_vstep)) * 0.0162162162;
    sum += texture2D(tex[i_zero], vec2(tc.x - 3.0 * blur_hstep, tc.y - 3.0 * blur_vstep)) * 0.0540540541;
    sum += texture2D(tex[i_zero], vec2(tc.x - 2.0 * blur_hstep, tc.y - 2.0 * blur_vstep)) * 0.1216216216;
    sum += texture2D(tex[i_zero], vec2(tc.x - 1.0 * blur_hstep, tc.y - 1.0 * blur_vstep)) * 0.1945945946;

    sum += texture2D(tex[i_zero], vec2(tc.x, tc.y)) * 0.2270270270;

    sum += texture2D(tex[i_zero], vec2(tc.x + 1.0 * blur_hstep, tc.y + 1.0 * blur_vstep)) * 0.1945945946;
    sum += texture2D(tex[i_zero], vec2(tc.x + 2.0 * blur_hstep, tc.y + 2.0 * blur_vstep)) * 0.1216216216;
    sum += texture2D(tex[i_zero], vec2(tc.x + 3.0 * blur_hstep, tc.y + 3.0 * blur_vstep)) * 0.0540540541;
    sum += texture2D(tex[i_zero], vec2(tc.x + 4.0 * blur_hstep, tc.y + 4.0 * blur_vstep)) * 0.0162162162;

    // discard alpha, multiply by vertex color and return
    gl_FragColor = v_color * vec4(sum.rgb, 1.0);
}
